# -*- coding: utf-8 -*-
"""
/***************************************************************************
Name                : Evaluation of environmental deterioration
Description         : Monitoring and evaluation system of environmental degradation and natural disasters.
Date                : 2014/02/14
Author              : Orkney, Inc.
email               : japan.sales@orkney.co.jp
Copyright           : (c) 2014 Orkney,Inc.
Original            : Geospatial Simulation , (C) 2011 by Dr. Kelly Thorp, USDA-ARS
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
from PyQt4.QtCore import pyqtSignature, Qt
from PyQt4.QtGui import QDialog, QMessageBox, QListWidgetItem, QApplication
from PyQt4.QtCore import QFileInfo, QVariant
from PyQt4.QtGui import QFileDialog
from qgis.core import QGis, QgsMapLayer, QgsMapLayerRegistry, QgsField, QgsSpatialIndex, QgsFeatureRequest
from qgis.core import QgsFeature, QgsVectorLayer
from qgis.core import QgsPoint, QgsRectangle, QgsGeometry, QgsVectorFileWriter
from Ui_EnvDeterioration import Ui_EnvDeterioration
import os

#環境劣化影響評価分析
class EnvDeterioration(QDialog):
    def __init__(self, iface): 
        QDialog.__init__(self) 
        # UIの呼び出し
        self.ui = Ui_EnvDeterioration()
        self.ui.setupUi(self)
        self.iface = iface
        self.mc = iface.mapCanvas() 
        #表示されているレイヤのみ選択可能とする
        self.layers = self.mc.layers()

        #UIの初期化
        self.ui.ProgressBar.setValue(0)
        self.ui.cmbBaseLayer.clear()
        self.outfile = ""
        self.costfile = ""
        
       #選択可能なレイヤの設定
        for i in self.layers:
            if i.type() == QgsMapLayer.VectorLayer:
                if i.geometryType() == QGis.Polygon:
                    self.ui.cmbBaseLayer.addItem(i.name(), i.id())

        bindex = self.ui.cmbBaseLayer.currentIndex()
        self.on_cmbBaseLayer_activated(bindex)

    #ポリゴンレイヤの選択
    @pyqtSignature("on_cmbBaseLayer_activated(int)")
    def on_cmbBaseLayer_activated(self, index):
        self.ui.ProgressBar.setValue(0)
        if self.ui.cmbBaseLayer.count() > 0:
            bid = self.ui.cmbBaseLayer.itemData(index)
            self.blayer = QgsMapLayerRegistry.instance().mapLayer(str(bid))

    #出力シェープファイルのブラウズ
    @pyqtSignature("on_btnBrowse_clicked()")
    def on_btnBrowse_clicked(self):
        if self.ui.cmbBaseLayer.count() > 0:            
            path = QFileInfo(self.blayer.source()).absolutePath()
        else:
            return             
            
        ofile = QFileDialog.getSaveFileName(self,
                                           u'Output shapefile',
                                           path,
                                           '*.shp') 
        
        if ofile != '':
            self.outfile = ofile
            self.ui.tbxOutput.setText(self.outfile)

    #コストパラメータファイルのブラウズ
    @pyqtSignature("on_btnBrowse_2_clicked()")
    def on_btnBrowse_2_clicked(self):            
        if self.ui.cmbBaseLayer.count() > 0:            
            path = QFileInfo(self.blayer.source()).absolutePath()
        else:
            return             
        ofile = QFileDialog.getOpenFileName(self,
                                           u'Cost parameter file',
                                           path,
                                           '*.*') 
        
        if ofile != '':
            self.costfile = ofile
            self.ui.tbxFile.setText(self.costfile)
            os.chdir(os.path.dirname(str(ofile)))

    #出力シェープファイルのテキスト直接入力
    @pyqtSignature("on_tbxOutput_textEdited(QString)")
    def on_tbxOutput_textEdited(self):
        if os.path.exists(self.ui.tbxOutput.text()):
            ofile = os.path.basename(str(self.ui.tbxOutput.text())) 
            QApplication.beep()
            res = QMessageBox.question(self,
                                       u'Save as a new file.',
                                       ofile + u' already exists.\n' + u'Overwrite?',
                                       QMessageBox.Yes,
                                       QMessageBox.No)
            if res == QMessageBox.Yes:
                self.outfile = self.ui.tbxOutput.text()
            else:
                self.ui.tbxOutput.setText(self.outfile)

    #コストパラメータファイルのテキスト直接入力
    @pyqtSignature("on_tbxFile_textEdited(QString)")
    def on_tbxFile_textEdited(self):
        self.costfile = self.ui.tbxFile.text()

    #実行ボタン処理
    @pyqtSignature("on_btnRun_clicked()")
    def on_btnRun_clicked(self):
                
        #入力項目のチェック
        if self.ui.cmbBaseLayer.count() < 1:
            QMessageBox.critical(self, u'Error', u'Select polygon layer.')
            return
      
        if self.costfile == '':
            QMessageBox.critical(self, u'Error', u'Specify cost parameter file.')
            return
      
        if self.outfile == '':
            QMessageBox.critical(self, u'Error', u'Specify output shapefile.')
            return
      
        if self.outfile == self.blayer.source():
            QMessageBox.critical(self, u'Error', u'Output shapefile is the same as input polygon file.')
            return
      
        #フィールド数の上限を越えないかチェック
        if  len(self.blayer.dataProvider().fields())  > 254:
            QMessageBox.critical(self,u'Error',u'Number of fields exceed the limit.')
            return
        
        #初期設定
        self.ui.ProgressBar.setFormat(u'Under calculation:%p%')
        self.ui.ProgressBar.setValue(0)
        self.setCursor(Qt.WaitCursor)

        #出力シェープファイルの作成（文字コードはポリゴンと同じ）
        bvlayer = QgsVectorLayer(self.blayer.source(), self.blayer.name(), self.blayer.providerType())
        result = QgsVectorFileWriter.writeAsVectorFormat(bvlayer, 
                                                         self.outfile, 
                                                         self.blayer.dataProvider().encoding(), 
                                                         self.blayer.crs())
        if result != QgsVectorFileWriter.NoError:
            self.setCursor(Qt.ArrowCursor)
            QMessageBox.critical(self,u'Error',u'Can not create shapefile.')
            return
        
        bolayer = QgsVectorLayer(self.outfile, self.blayer.name(), self.blayer.providerType())
        bprovider = bolayer.dataProvider()
        
        #計算結果を入れるTOTALフィールドの追加
        costfield = [QgsField("TOTAL", QVariant.Double)]
        bprovider.addAttributes(costfield)
        
        #元の数値フィールドの取得
        bfields = bprovider.fields()
        basefld = []
        for fld in bfields.toList():
            if fld.typeName() != 'String': #Numeric fields
                basefld.append(fld.name())

        #コストパラメータファイルから入力
        #コストパラメータファイルは "カテゴリーフィールド名,コスト" のレコードで構成される
        #元のフィールドにないカテゴリーフィールドの値は計算から除く
        if not os.path.exists(self.costfile):
            self.setCursor(Qt.ArrowCursor)
            QMessageBox.critical(self,u'Error',u'Cost parameter file does not exist.')
            return
        else:
            costfields = []
            costs = []
            with open(self.costfile) as reader:
                for line in reader:
                    items = line.split(',')
                    for bfld in basefld:
                        if bfld == str(items[0]):
                            costfields.append(str(items[0]))
                            costs.append(float(items[1]))
                            break

        #コストの計算
        len2 = len(bfields)  - 1
        b1 = 0
        b2 = bprovider.featureCount()
        attr={}
        for bfeat in bprovider.getFeatures():
            b1+=1
            attr.clear()
            total = 0
            for i in range(len(costfields)):
                value = float(bfeat.attribute(costfields[i]))
                cost = value * costs[i]
                total += cost
            attr.update({len2:total})
            result = bprovider.changeAttributeValues({bfeat.id():attr})
            if not result:
                self.setCursor(Qt.ArrowCursor)
                QMessageBox.critical(self, u'Error', u'Can not update property.')
                return           

            self.ui.ProgressBar.setValue(float(b1)/float(b2) * 100.0)
            QApplication.processEvents()

        #計算終了後、結果を地図に表示するかしないかを選択する
        name = QFileInfo(self.outfile).completeBaseName()
        result = QMessageBox.question(self,u'Finish calculation.',
                                      u'Display output shapefile on map？',
                                      QMessageBox.Yes,
                                      QMessageBox.No)
        if result == QMessageBox.Yes:
            self.iface.addVectorLayer(self.outfile, name, 'ogr')

        self.setCursor(Qt.ArrowCursor)
        self.close()
                 
    @pyqtSignature("on_btnExit_clicked()")
    def on_btnExit_clicked(self):
        self.close() 