# -*- coding: utf-8 -*-
"""
/***************************************************************************
Name                : Evaluation of environmental deterioration
Description         : Monitoring and evaluation system of environmental degradation and natural disasters.
Date                : 2014/02/14
Author              : Orkney, Inc.
email               : japan.sales@orkney.co.jp
Copyright           : (c) 2014 Orkney,Inc.
Original            : Geospatial Simulation , (C) 2011 by Dr. Kelly Thorp, USDA-ARS
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
from PyQt4.QtCore import QObject, QSettings
from PyQt4.QtGui import QAction, QIcon, QMessageBox
import os
import sys
import resources
from OMstatics import OMstatics
from EnvDeterioration import EnvDeterioration

class EnviromentalEvaluation: 

    def __init__(self, iface):
        self.iface = iface
        self.plugin_dir = os.path.dirname(__file__)
        locale = QSettings().value("locale/userLocale")[0:2]
        localePath = os.path.join(self.plugin_dir, 'i18n', 'envronmentalevaluation_{}.qm'.format(locale))
        if os.path.exists(localePath):
            self.translator = QTranslator()
            self.translator.load(localePath)
            if qVersion() > '4.3.3':
                QCoreApplication.installTranslator(self.ttranslator)

    def initGui(self):  
        # プラグインの初期設定とactionの作成
        icon_om = QIcon(":/plugins/EnviromentalEvaluation/icon_om.png")
        self.omstat = QAction(icon_om,u"Overlay and Matrix tabulation", self.iface.mainWindow())
        icon_ed = QIcon(":/plugins/EnviromentalEvaluation/icon_ed.png")
        self.envdet = QAction(icon_ed,u"Environment Deterioration Analysis", self.iface.mainWindow())
        
        # actionの接続
        self.omstat.triggered.connect(self.OMstat)
        self.envdet.triggered.connect(self.EnvDet)
         
        # プラグインメニューバーとツールバー（アイコン）への追加        
        self.iface.addPluginToMenu(u"Environment Deterioration Evaluation", self.omstat)
        self.iface.addPluginToMenu(u"Environment Deterioration Evaluation", self.envdet)
        self.iface.addToolBarIcon(self.omstat)
        self.iface.addToolBarIcon(self.envdet)

    def unload(self):
        # プラグインの削除
        self.iface.removePluginMenu(u"Environment Deterioration Evaluation", self.omstat)
        self.iface.removePluginMenu(u"Environment Deterioration Evaluation", self.envdet)
        self.iface.removeToolBarIcon(self.omstat)
        self.iface.removeToolBarIcon(self.envdet)
        
    # オーバーレイ＆マトリクス集計の実行設定
    def OMstat(self): 
        dlg = OMstatics(self.iface) 
        dlg.exec_() 
        
    # 環境劣化影響評価の実行設定
    def EnvDet(self):
        dlg = EnvDeterioration(self.iface)
        dlg.exec_()
