# -*- coding: utf-8 -*-
"""
/***************************************************************************
Name                : Evaluation of environmental deterioration
Description         : Monitoring and evaluation system of environmental degradation and natural disasters.
Date                : 2015/04/6
Author              : Orkney, Inc.
email               : japan.sales@orkney.co.jp
Copyright           : (c) 2015 Orkney,Inc.
Original            : Geospatial Simulation , (C) 2011 by Dr. Kelly Thorp, USDA-ARS
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
from PyQt4.QtCore import pyqtSignature, Qt
from PyQt4.QtGui import QDialog, QMessageBox, QListWidgetItem, QApplication
from PyQt4.QtCore import QFileInfo, QVariant
from PyQt4.QtGui import QFileDialog
from qgis.core import QGis, QgsMapLayer, QgsMapLayerRegistry, QgsField, QgsSpatialIndex, QgsFeatureRequest
from qgis.core import QgsFeature, QgsVectorLayer
from qgis.core import QgsPoint, QgsRectangle, QgsGeometry, QgsVectorFileWriter
from collections import defaultdict
import qgis.utils
import os
from Ui_OMstatics import Ui_OMstatics

#オーバーレイ＆マトリクス集計
class OMstatics(QDialog):
    def __init__(self, iface): 
        QDialog.__init__(self) 
        # UIの呼び出し
        self.ui = Ui_OMstatics()
        self.ui.setupUi(self)
        self.iface = iface
        self.mc = iface.mapCanvas() 
        #表示されているレイヤのみ選択可能とする
        self.layers = self.mc.layers()

        #UIの初期化
        self.ui.ProgressBar.setValue(0)
        self.ui.cmbBaseLayer.clear()
        self.ui.cmbProcessLayer.clear()
        self.outfile = ""

       #選択可能なレイヤの設定
        for i in self.layers:
            if i.type() == QgsMapLayer.VectorLayer:
                if i.geometryType() == QGis.Polygon:
                    self.ui.cmbBaseLayer.addItem(i.name(), i.id())
            elif i.type() == QgsMapLayer.RasterLayer:
                self.ui.cmbProcessLayer.addItem(i.name(), i.id())
 
        bindex = self.ui.cmbBaseLayer.currentIndex()
        self.on_cmbBaseLayer_activated(bindex)
        
        pindex = self.ui.cmbProcessLayer.currentIndex()    
        self.on_cmbProcessLayer_activated(pindex)  

    #ポリゴンレイヤの選択
    @pyqtSignature("on_cmbBaseLayer_activated(int)")
    def on_cmbBaseLayer_activated(self, index):
        self.ui.ProgressBar.setValue(0)
        if self.ui.cmbBaseLayer.count() > 0:
            bid = self.ui.cmbBaseLayer.itemData(index)
            self.blayer = QgsMapLayerRegistry.instance().mapLayer(str(bid))
    
    #ラスタレイヤの選択
    @pyqtSignature("on_cmbProcessLayer_activated(int)")
    def on_cmbProcessLayer_activated(self, index):
        self.ui.ProgressBar.setValue(0)
        if self.ui.cmbProcessLayer.count() > 0:
            pid = self.ui.cmbProcessLayer.itemData(index)
            self.player = QgsMapLayerRegistry.instance().mapLayer(str(pid))
    
    #出力シェープファイルのブラウズ
    @pyqtSignature("on_btnBrowse_clicked()")
    def on_btnBrowse_clicked(self):
        if self.ui.cmbProcessLayer.count() > 0:            
            path = QFileInfo(self.blayer.source()).absolutePath()
        else:
            return
            
        ofile = QFileDialog.getSaveFileName(self,
                                           u'Output shapefile',
                                           path,
                                           '*.shp') 
        
        if ofile != '':
            self.outfile = ofile
        else:
            self.outfile = ''
        self.ui.tbxOutput.setText(self.outfile)
        
    #出力シェープファイルのテキスト直接入力
    @pyqtSignature("on_tbxOutput_textEdited(QString)")
    def on_tbxOutput_textEdited(self):
        if os.path.exists(self.ui.tbxOutput.text()):
            ofile = os.path.basename(str(self.ui.tbxOutput.text())) 
            QApplication.beep()
            res = QMessageBox.question(self,
                                       u'Save as a new file.',
                                       ofile + u' already exists.\n' + u'Overwrite?',
                                       QMessageBox.Yes,
                                       QMessageBox.No)
            if res == QMessageBox.Yes:
                self.outfile = self.ui.tbxOutput.text()
            else:
                self.ui.tbxOutput.setText(self.outfile)

    #実行ボタン処理
    @pyqtSignature("on_btnRun_clicked()")
    def on_btnRun_clicked(self):
        
        QgisVersion = qgis.utils.QGis.QGIS_VERSION_INT
        
        #入力項目のチェック
        if self.ui.cmbProcessLayer.count() < 1:
            QMessageBox.critical(self, u'Error', u'Select raster layer.')
            return
      
        if self.ui.cmbBaseLayer.count() < 1:
            QMessageBox.critical(self, u'Error', u'Select polygon layer.')
            return
      
        if self.outfile == '':
            QMessageBox.critical(self, u'Error', u'Specify output shapefile.')
            return
      
        if self.outfile == self.blayer.source():
            QMessageBox.critical(self, u'Error', u'Output shapefile is the same as input polygon file.')
            return
      
        #初期設定
        self.ui.ProgressBar.setFormat(u'Converting raster to polygon:%p%')
        self.ui.ProgressBar.setValue(0)
        self.setCursor(Qt.WaitCursor)
        toPoint = False

        #ラスタから変換
        #ラスタレイヤ情報の取得
        numX = self.player.width()
        numY = self.player.height()
        pixsizeX = self.player.rasterUnitsPerPixelX()
        pixsizeY = self.player.rasterUnitsPerPixelY()
        extents = self.player.extent()
        LLX = extents.xMinimum()
        LLY = extents.yMinimum()
        URX = extents.xMaximum()
        URY = extents.yMaximum()
        rprovider = self.player.dataProvider()
            
        if LLX > URX or LLY > URY:
            self.setCursor(Qt.ArrowCursor)
            QMessageBox.critical(self,u'Error',u'Raster data is invalid.')
            return
        
        #ポリゴンレイヤ情報の取得
        bextents = self.blayer.extent()
        bLLX = bextents.xMinimum()
        bLLY = bextents.yMinimum()
        bURX = bextents.xMaximum()
        bURY = bextents.yMaximum()

       #ラスタから変換する範囲の計算
        Xmin = bLLX
        if Xmin < LLX:
            Xmin = LLX
        Ymin = bLLY
        if Ymin < LLY:
            Ymin = LLY
        Xmax= bURX
        if Xmax > URX:
            Xmax= URX
        Ymax= bURY
        if Ymax > URY:
            Ymax= URY
        XStart = int((Xmin - LLX - pixsizeX/2.0) / pixsizeX) - 1
        if XStart < 0:
            XStart = 0
        YStart = int((Ymin - LLY - pixsizeY/2.0) / pixsizeY)  - 1
        if YStart < 0:
            YStart = 0
        XEnd   = int((Xmax - LLX + pixsizeX/2.0) / pixsizeX) + 1
        if XEnd - XStart  > numX:
            XEnd = numX
        YEnd   = int((Ymax - LLY + pixsizeY/2.0) / pixsizeY) + 1
        if YEnd - YStart  > numY:
            YEnd = numY
        numX = XEnd - XStart
        numY = YEnd - YStart


        #ピクセルの中心座標からフィーチャの座標を計算して変換
        pfeat = QgsFeature()
        crsid = str(self.player.crs().authid())              
        if toPoint:        
            pvlayer = QgsVectorLayer("Point?crs="+crsid, "templayer", "memory")
        else:
            pvlayer = QgsVectorLayer("Polygon?crs="+crsid, "templayer", "memory")
        fields = [QgsField("ID", QVariant.Int)]
        for i in range(self.player.bandCount()):
            #ラスタデータの値はB + バンド番号(B0)にフィールドに格納
            bandname = "B" + str(i)
            #値は整数値(Int)
            fields.append(QgsField(bandname, QVariant.Int))
        pprovider = pvlayer.dataProvider()
        pprovider.addAttributes(fields)
        count = 0
        attr = []
        totpix = float(numX*numY)
        #1点すつ変換
        for y in range(YStart,YEnd):
            pvlayer.startEditing()
            for x in range(XStart,XEnd):       
                newpt = QgsPoint(LLX+x*pixsizeX+pixsizeX/2.0,
                                 LLY+y*pixsizeY+pixsizeY/2.0)
                data = rprovider.identify(newpt, 1).results()

                if toPoint:
                    pfeat.setGeometry(QgsGeometry.fromPoint(newpt))                
                else:
                    newrect = QgsRectangle(LLX+x*pixsizeX,
                                           LLY+y*pixsizeY,
                                           LLX+x*pixsizeX+pixsizeX,
                                           LLY+y*pixsizeY+pixsizeY)
                    pfeat.setGeometry(QgsGeometry.fromRect(newrect))
                attr = list(data.values())
                attr.insert(0,count)
                pfeat.setAttributes(attr) 
                result = pvlayer.addFeature(pfeat)
                if not result:
                    self.setCursor(Qt.ArrowCursor)
                    QMessageBox.critical(self,u'Error',u'Can not convert to polygon.')
                    return        
                del newpt
                if not toPoint:
                    del newrect
                count+=1
                self.ui.ProgressBar.setValue(float(count)/totpix * 100.0)
                QApplication.processEvents()  
            pvlayer.commitChanges()
        pvlayer.updateExtents()

        #２つのレイヤから集計計算
        data = []
        self.ui.ProgressBar.setFormat(u'Under calculation:%p%')
        self.ui.ProgressBar.setValue(0)

        #変換したレイヤの値からカテゴリー分類
        categ = []
        fname = "B0"
        ncat = 0
        for pfeat in pprovider.getFeatures():
            if not pfeat.attribute(fname) and pfeat.attribute(fname) != 0:
                continue
            value = int(pfeat.attribute(fname))
            if ncat == 0:
                categ.append(value)
                ncat = 1
            else:
                addcat = 1
                for i in categ:
                    if value == i:
                        addcat = 0
                        break
                if addcat == 1:
                    categ.append(value)
                    ncat += 1
        categ.sort()

        #フィールド数の上限を越えないかチェック
        if  len(self.blayer.dataProvider().fields()) + len(categ) > 255:
            self.setCursor(Qt.ArrowCursor)
            QMessageBox.critical(self,u'Error',u'Number of fields exceed the limit.')
            return
        
        #出力シェープファイルの作成（文字コードはポリゴンと同じ）
        pprovider = pvlayer.dataProvider()
        bvlayer = QgsVectorLayer(self.blayer.source(), self.blayer.name(), self.blayer.providerType())
        writer = QgsVectorFileWriter.writeAsVectorFormat(bvlayer, 
                                                         self.outfile, 
                                                         self.blayer.dataProvider().encoding(), 
                                                         self.blayer.crs())
        if writer != QgsVectorFileWriter.NoError:
            self.setCursor(Qt.ArrowCursor)
            QMessageBox.critical(self,u'Error',u'Can not create shapefile.')
            return

        bolayer = QgsVectorLayer(self.outfile, self.blayer.name(), self.blayer.providerType())
        bprovider = bolayer.dataProvider()

        #カテゴリーのフィールドを追加 フィールド名は C + カテゴリー値
        fields = []
        for catvalue in categ:
            fields.append(QgsField("C"+str(catvalue), QVariant.Int))
        bprovider.addAttributes(fields)

        #空間インデックスの作成
        spindex = QgsSpatialIndex()
        for pfeat in pprovider.getFeatures():
            spindex.insertFeature(pfeat)
            
        #ベースとなるポリゴンに含まれるフィーチャを選択
        featreq = QgsFeatureRequest()
        bfields = bprovider.fields()
        len2 = len(bfields) - len(categ)
        b1 = 0
        b2 = bprovider.featureCount()
        attr = {}
        for bfeat in bprovider.getFeatures():
            b1+= 1
            attr.clear()
            bgeom = bfeat.geometry()                                          
            intersect = spindex.intersects(bgeom.boundingBox())
            data[:] = []
            if QgisVersion < 20300:
                for fid in intersect:
                    pfeat = pvlayer.getFeatures(featreq.setFilterFid(fid)).next()
                    if pfeat.geometry().intersects(bgeom) == False:
                        data.append(fid)           
            else:
                request = featreq.setFilterFids(intersect);
                pfeats = pvlayer.getFeatures(request)
                for pfeat in pfeats:
                    if pfeat.geometry().intersects(bgeom) == False:
                        data.append(pfeat.id())           
            for fid in data:        
                intersect.pop(intersect.index(fid))
                              
            #カテゴリー値毎にフィーチャ数をカウントして値を代入
            pfeats = []
            if QgisVersion < 20300:
                for fid in intersect:
                    pfeats.append(pvlayer.getFeatures(featreq.setFilterFid(fid)).next())
            else:
                request = featreq.setFilterFids(intersect)
                for pfeat in  pvlayer.getFeatures(request):
                    pfeats.append(pfeat)

            catvalue = defaultdict(int)
            for pfeat in pfeats:
                if not pfeat.attribute(fname) and pfeat.attribute(fname) != 0:
                    continue
                catvalue[int(pfeat.attribute(fname))] += 1

            icat = 0
            for bitem in categ:
                attr.update({(len2+icat):catvalue[bitem]})
                icat += 1
            result = bprovider.changeAttributeValues({bfeat.id():attr})
            if not result:
                self.setCursor(Qt.ArrowCursor)
                QMessageBox.critical(self, u'Error', u'Can not update property.')
                return           

            self.ui.ProgressBar.setValue(float(b1)/float(b2) * 100.0)
            QApplication.processEvents()

        del writer

        #計算終了後、結果を地図に表示するかしないかを選択する
        name = QFileInfo(self.outfile).completeBaseName()
        result = QMessageBox.question(self,u'Finish calculation.',
                                      u'Display output shapefile on map？',
                                      QMessageBox.Yes,
                                      QMessageBox.No)
        if result == QMessageBox.Yes:
            self.iface.addVectorLayer(self.outfile, name, 'ogr')

        self.setCursor(Qt.ArrowCursor)
        self.close()

    #閉じるボタンを押したときの処理
    @pyqtSignature("on_btnExit_clicked()")
    def on_btnExit_clicked(self):
        self.close()
